/*****************************************************************************
** FILE IDENTIFICATION
**
**   Name:          dialogs.cpp
**   Purpose:       Dialog routines for CTSim program
**   Programmer:    Kevin Rosenberg
**   Date Started:  July 2000
**
**  This is part of the CTSim program
**  Copyright (C) 1983-2000 Kevin Rosenberg
**
**  $Id: dialogs.cpp,v 1.21 2000/12/27 03:16:02 kevin Exp $
**
**  This program is free software; you can redistribute it and/or modify
**  it under the terms of the GNU General Public License (version 2) as
**  published by the Free Software Foundation.
**
**  This program is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
**  GNU General Public License for more details.
**
**  You should have received a copy of the GNU General Public License
**  along with this program; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
******************************************************************************/

#ifdef __GNUG__
// #pragma implementation
#endif

// For compilers that support precompilation, includes "wx/wx.h".
#include "wx/wxprec.h"

#ifdef __BORLANDC__
#pragma hdrstop
#endif

#ifndef WX_PRECOMP
#include "wx/wx.h"
#endif

#if !wxUSE_DOC_VIEW_ARCHITECTURE
#error You must set wxUSE_DOC_VIEW_ARCHITECTURE to 1 in setup.h!
#endif

#include "wx/statline.h"
#include "wx/sizer.h"
#include "dialogs.h"
#include "ctsim.h"
#include "ct.h"
//#include "ctsupport.h"
//#include "scanner.h"
//#include "phantom.h"
//#include "filter.h"
//#include "backprojectors.h"
#include "docs.h"
#include "views.h"
#include "imagefile.h"

#if defined(MSVC) || HAVE_SSTREAM
#include <sstream>
#else
#include <sstream_subst>
#endif


///////////////////////////////////////////////////////////////////////
// CLASS IMPLEMENTATION
//    StringValueAndTitleListBox
///////////////////////////////////////////////////////////////////////

StringValueAndTitleListBox::StringValueAndTitleListBox (wxDialog* pParent, int nChoices, const char* aszTitle[], const char* aszValue[])
: wxListBox ()
{
  wxString* psTitle = new wxString [nChoices];
  for (int i = 0; i < nChoices; i++)
    psTitle[i] = aszTitle[i];
  
  Create (pParent, -1, wxDefaultPosition, wxSize(200,-1), nChoices, psTitle, wxLB_SINGLE | wxLB_NEEDED_SB);
  
  m_ppszValues = aszValue;
  delete [] psTitle;
};

const char*
StringValueAndTitleListBox::getSelectionStringValue (void) const
{
  return m_ppszValues[GetSelection()];
}


///////////////////////////////////////////////////////////////////////
// CLASS IMPLEMENTATION
//    DialogGetPhantom
///////////////////////////////////////////////////////////////////////

DialogGetPhantom::DialogGetPhantom (wxFrame* pParent, int iDefaultPhantom)
: wxDialog (pParent, -1, "Select Phantom", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, "Select Phantom"), 0, wxALIGN_CENTER | wxALL, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxALL, 5);
  
  m_pListBoxPhantom = new StringValueAndTitleListBox (this, Phantom::getPhantomCount(), Phantom::getPhantomTitleArray(), Phantom::getPhantomNameArray());
  m_pListBoxPhantom->SetSelection (iDefaultPhantom);
  pTopSizer->Add (m_pListBoxPhantom, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

const char*
DialogGetPhantom::getPhantom(void)
{
  return m_pListBoxPhantom->getSelectionStringValue();
}


///////////////////////////////////////////////////////////////////////
// CLASS IMPLEMENTATION
//    DialogGetComparisonImage
///////////////////////////////////////////////////////////////////////

DialogGetComparisonImage::DialogGetComparisonImage (wxFrame* pParent, const char* const pszTitle, const std::vector<ImageFileDocument*>& rVecIF, bool bShowMakeDifference)
: wxDialog (pParent, -1, pszTitle, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION), m_rVecIF(rVecIF)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, pszTitle), 0, wxALIGN_CENTER | wxALL, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxALL, 5);
  
  int iNImages = m_rVecIF.size();
  wxString* pstrImages = new wxString [iNImages];
  for (int i = 0; i < iNImages; i++) {
    ImageFileView* pView = dynamic_cast<ImageFileView*>(m_rVecIF[i]->GetFirstView());
    if (pView)
      pstrImages[i] = pView->getFrame()->GetTitle();
  }

  m_pListBoxImageChoices = new wxListBox (this, -1, wxDefaultPosition, wxDefaultSize, iNImages, pstrImages, wxLB_SINGLE);
  delete [] pstrImages;

  m_pListBoxImageChoices->SetSelection (0);
  pTopSizer->Add (m_pListBoxImageChoices, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  
  if (bShowMakeDifference) {
    m_pMakeDifferenceImage = new wxCheckBox (this, -1, "Make Difference Image");
    m_pMakeDifferenceImage->SetValue (FALSE);
    pTopSizer->Add (m_pMakeDifferenceImage, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  } else
    m_pMakeDifferenceImage = NULL;

  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

ImageFileDocument*
DialogGetComparisonImage::getImageFileDocument(void)
{
  return m_rVecIF[ m_pListBoxImageChoices->GetSelection() ];
}

bool
DialogGetComparisonImage::getMakeDifferenceImage()
{
  if (m_pMakeDifferenceImage)
    return m_pMakeDifferenceImage->GetValue();
  else
    return false;
}


/////////////////////////////////////////////////////////////////////
// CLASS DiaglogGetMinMax Implementation
/////////////////////////////////////////////////////////////////////

DialogGetMinMax::DialogGetMinMax (wxFrame* pParent, const char* const pszTitle, double dDefaultMin, double dDefaultMax)
: wxDialog (pParent, -1, pszTitle, wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, pszTitle), 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  std::ostringstream os;
  os << dDefaultMin;
  m_pTextCtrlMin = new wxTextCtrl (this, -1, os.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osMax;
  osMax << dDefaultMax;
  m_pTextCtrlMax = new wxTextCtrl (this, -1, osMax.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  
  wxFlexGridSizer *pGridSizer = new wxFlexGridSizer (2);
  pGridSizer->Add (new wxStaticText (this, -1, "Minimum"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlMin, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Maximum"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlMax, 0, wxALIGN_CENTER_VERTICAL);
  pTopSizer->Add (pGridSizer, 1, wxALL, 10);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

DialogGetMinMax::~DialogGetMinMax (void)
{
}

double
DialogGetMinMax::getMinimum (void)
{
  wxString strCtrl = m_pTextCtrlMin->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return dValue;
  else
    return (m_dDefaultMin);
}

double
DialogGetMinMax::getMaximum (void)
{
  wxString strCtrl = m_pTextCtrlMax->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return dValue;
  else
    return (m_dDefaultMax);
}


/////////////////////////////////////////////////////////////////////
// CLASS DialogAutoScaleParameters IMPLEMENTATION
/////////////////////////////////////////////////////////////////////

DialogAutoScaleParameters::DialogAutoScaleParameters (wxFrame *pParent, double mean, double mode, double median, double stddev, double dDefaultScaleFactor)
: wxDialog (pParent, -1, "Auto Scale Parameters", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION), m_dMean(mean), m_dMode(mode), m_dMedian(median), m_dStdDev(stddev)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, "Auto Scale Parameters"), 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxString asTitle[3];
  asTitle[0] = "Median";
  asTitle[1] = "Mode";
  asTitle[2] = "Mean";
  
  m_pListBoxCenter = new wxListBox (this, -1, wxDefaultPosition, wxDefaultSize, 3, asTitle, wxLB_SINGLE | wxLB_NEEDED_SB);
  m_pListBoxCenter->SetSelection (0);
  pTopSizer->Add (m_pListBoxCenter, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  
  wxGridSizer *pGridSizer = new wxGridSizer (2);
  pGridSizer->Add (new wxStaticText (this, -1, "Standard Deviation Factor"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  std::ostringstream osDefaultFactor;
  osDefaultFactor << dDefaultScaleFactor;
  m_pTextCtrlStdDevFactor = new wxTextCtrl (this, -1, osDefaultFactor.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  pGridSizer->Add (m_pTextCtrlStdDevFactor, 0, wxALIGN_CENTER_VERTICAL);
  pTopSizer->Add (pGridSizer, 1, wxALL, 10);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

bool
DialogAutoScaleParameters::getMinMax (double* pMin, double* pMax)
{
  int iCenter = m_pListBoxCenter->GetSelection();
  double dCenter = m_dMedian;
  if (iCenter == 1)
    dCenter = m_dMode;
  else if (iCenter == 2)
    dCenter = m_dMode;
  
  wxString sStddevFactor = m_pTextCtrlStdDevFactor->GetValue();
  double dValue;
  if (! sStddevFactor.ToDouble (&dValue)) {
    *theApp->getLog() << "Error: Non-numeric Standard Deviation Factor of " << sStddevFactor << "\n";
    return false;
  }
  double dHalfWidth = dValue * m_dStdDev / 2;
  *pMin = dCenter - dHalfWidth;
  *pMax = dCenter + dHalfWidth;
  *theApp->getLog() << "Setting minimum to " << *pMin << " and maximum to " << *pMax << "\n";
  
  return true;
}

double
DialogAutoScaleParameters::getAutoScaleFactor ()
{
  wxString sStddevFactor = m_pTextCtrlStdDevFactor->GetValue();
  double dValue = 1.;
  if (! sStddevFactor.ToDouble (&dValue)) {
    *theApp->getLog() << "Error: Non-numeric Standard Deviation Factor of " << sStddevFactor << "\n";
  }
  
  return dValue;
}


/////////////////////////////////////////////////////////////////////
// CLASS IDENTIFICATION
//
// DialogGetRasterParameters
/////////////////////////////////////////////////////////////////////

DialogGetRasterParameters::DialogGetRasterParameters (wxFrame* pParent, int iDefaultXSize, int iDefaultYSize, int iDefaultNSamples)
: wxDialog (pParent, -1, "Set Rasterization Parameters", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, "Set Rasterization Parameters"), 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  std::ostringstream os;
  os << iDefaultXSize;
  m_pTextCtrlXSize = new wxTextCtrl (this, -1, os.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osYSize;
  osYSize << iDefaultYSize;
  m_pTextCtrlYSize = new wxTextCtrl (this, -1, osYSize.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osNSamples;
  osNSamples << iDefaultNSamples;
  m_pTextCtrlNSamples = new wxTextCtrl (this, -1, osNSamples.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  
  wxFlexGridSizer *pGridSizer = new wxFlexGridSizer (2);
  pGridSizer->Add (new wxStaticText (this, -1, "X Size"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlXSize, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Y Size"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlYSize, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Samples per Pixel"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlNSamples, 0, wxALIGN_CENTER_VERTICAL);
  pTopSizer->Add (pGridSizer, 1, wxALL, 10);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

DialogGetRasterParameters::~DialogGetRasterParameters (void)
{
}


unsigned int
DialogGetRasterParameters::getXSize (void)
{
  wxString strCtrl = m_pTextCtrlXSize->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultXSize);
}

unsigned int
DialogGetRasterParameters::getYSize (void)
{
  wxString strCtrl = m_pTextCtrlYSize->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultYSize);
}


unsigned int
DialogGetRasterParameters::getNSamples (void)
{
  wxString strCtrl = m_pTextCtrlNSamples->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultNSamples);
}



/////////////////////////////////////////////////////////////////////
// CLASS IDENTIFICATION
//
// DialogGetProjectionParameters
/////////////////////////////////////////////////////////////////////


DialogGetProjectionParameters::DialogGetProjectionParameters (wxFrame* pParent, int iDefaultNDet, int iDefaultNView, int iDefaultNSamples, double dDefaultRotAngle, double dDefaultFocalLength, double dDefaultFieldOfView, int iDefaultGeometry, int iDefaultTrace)
: wxDialog (pParent, -1, "Set Projection Parameters", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  m_dDefaultRotAngle = dDefaultRotAngle;
  m_dDefaultFocalLength = dDefaultFocalLength;
  m_dDefaultFieldOfView = dDefaultFieldOfView;
  m_iDefaultNSamples = iDefaultNSamples;
  m_iDefaultNView = iDefaultNView;
  m_iDefaultNDet = iDefaultNDet;
  m_iDefaultTrace = iDefaultTrace;
  m_iDefaultGeometry = iDefaultGeometry;
  
  pTopSizer->Add (new wxStaticText (this, -1, "Set Projection Parameters"), 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  std::ostringstream os;
  os << iDefaultNDet;
  m_pTextCtrlNDet = new wxTextCtrl (this, -1, os.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osNView;
  osNView << iDefaultNView;
  m_pTextCtrlNView = new wxTextCtrl (this, -1, osNView.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osNSamples;
  osNSamples << iDefaultNSamples;
  m_pTextCtrlNSamples = new wxTextCtrl (this, -1, osNSamples.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osRotAngle;
  osRotAngle << dDefaultRotAngle;
  m_pTextCtrlRotAngle = new wxTextCtrl (this, -1, osRotAngle.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osFocalLength;
  osFocalLength << dDefaultFocalLength;
  m_pTextCtrlFocalLength = new wxTextCtrl (this, -1, osFocalLength.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osFieldOfView;
  osFieldOfView << dDefaultFieldOfView;
  m_pTextCtrlFieldOfView = new wxTextCtrl (this, -1, osFieldOfView.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  
  wxFlexGridSizer* pGridSizer = new wxFlexGridSizer (2);
  pGridSizer->Add (new wxStaticText (this, -1, "Scanner Geometry"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  m_pListBoxGeometry = new StringValueAndTitleListBox (this, Scanner::getGeometryCount(), Scanner::getGeometryTitleArray(), Scanner::getGeometryNameArray());
  m_pListBoxGeometry->SetSelection (iDefaultGeometry);
  
  pGridSizer->Add (m_pListBoxGeometry, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  
  pGridSizer->Add (new wxStaticText (this, -1, "Detectors"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlNDet, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Views"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlNView, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Samples per Detector"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlNSamples, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Rotation Angle (PI units)"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlRotAngle, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Focal Length Ratio (phantom radius units)"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlFocalLength, 0, wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Field of View (phantom diameter units)"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlFieldOfView, 0, wxALIGN_CENTER_VERTICAL);
  
  m_pListBoxTrace = new StringValueAndTitleListBox (this, Trace::getTraceCount(), Trace::getTraceTitleArray(), Trace::getTraceNameArray());
  m_pListBoxTrace->SetSelection (iDefaultTrace);
  
  pGridSizer->Add (new wxStaticText (this, -1, "Trace Level"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pListBoxTrace, 0, wxALL | wxALIGN_CENTER | wxEXPAND);
  pTopSizer->Add (pGridSizer, 1, wxALL, 10);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

DialogGetProjectionParameters::~DialogGetProjectionParameters (void)
{
}


unsigned int
DialogGetProjectionParameters::getNDet (void)
{
  wxString strCtrl = m_pTextCtrlNDet->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultNDet);
}

unsigned int
DialogGetProjectionParameters::getNView (void)
{
  wxString strCtrl = m_pTextCtrlNView->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultNView);
}


unsigned int
DialogGetProjectionParameters::getNSamples (void)
{
  wxString strCtrl = m_pTextCtrlNSamples->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultNSamples);
}

double
DialogGetProjectionParameters::getRotAngle (void)
{
  wxString strCtrl = m_pTextCtrlRotAngle->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return (dValue * PI);
  else
    return (m_dDefaultRotAngle);
}

double
DialogGetProjectionParameters::getFocalLengthRatio (void)
{
  wxString strCtrl = m_pTextCtrlFocalLength->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return (dValue);
  else
    return (m_dDefaultFocalLength);
}

double
DialogGetProjectionParameters::getFieldOfViewRatio (void)
{
  wxString strCtrl = m_pTextCtrlFieldOfView->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return (dValue);
  else
    return (m_dDefaultFieldOfView);
}

const char*
DialogGetProjectionParameters::getGeometry (void)
{
  return m_pListBoxGeometry->getSelectionStringValue();
}

int
DialogGetProjectionParameters::getTrace (void)
{
  return Trace::convertTraceNameToID(m_pListBoxTrace->getSelectionStringValue());
}



/////////////////////////////////////////////////////////////////////
// CLASS IDENTIFICATION
//
// DialogGetReconstructionParameters
/////////////////////////////////////////////////////////////////////


DialogGetReconstructionParameters::DialogGetReconstructionParameters (wxFrame* pParent, int iDefaultXSize, int iDefaultYSize, int iDefaultFilterID, double dDefaultFilterParam,  int iDefaultFilterMethodID, int iDefaultFilterGenerationID, int iDefaultZeropad, int iDefaultInterpID, int iDefaultInterpParam, int iDefaultBackprojectID, int iTrace)
: wxDialog (pParent, -1, "Set Reconstruction Parameters", wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxCAPTION)
{
  wxBoxSizer* pTopSizer = new wxBoxSizer (wxVERTICAL);
  
  pTopSizer->Add (new wxStaticText (this, -1, "Set Reconstruction Parameters"), 0, wxALIGN_CENTER | wxTOP | wxLEFT | wxRIGHT, 5);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  std::ostringstream os;
  os << iDefaultXSize;
  m_pTextCtrlXSize = new wxTextCtrl (this, -1, os.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osYSize;
  osYSize << iDefaultYSize;
  m_pTextCtrlYSize = new wxTextCtrl (this, -1, osYSize.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osFilterParam;
  osFilterParam << dDefaultFilterParam;
  m_pTextCtrlFilterParam = new wxTextCtrl (this, -1, osFilterParam.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osZeropad;
  osZeropad << iDefaultZeropad;
  m_pTextCtrlZeropad = new wxTextCtrl (this, -1, osZeropad.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  std::ostringstream osInterpParam;
  osInterpParam << iDefaultInterpParam;
  m_pTextCtrlInterpParam = new wxTextCtrl (this, -1, osInterpParam.str().c_str(), wxDefaultPosition, wxSize(100, 25), 0);
  
  wxFlexGridSizer* pGridSizer = new wxFlexGridSizer (6);
  pGridSizer->Add (new wxStaticText (this, -1, "Filter"), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
  m_pListBoxFilter = new StringValueAndTitleListBox (this, SignalFilter::getFilterCount(), SignalFilter::getFilterTitleArray(), SignalFilter::getFilterNameArray());
  m_pListBoxFilter->SetSelection (iDefaultFilterID);
  pGridSizer->Add (m_pListBoxFilter, 0, wxALL | wxALIGN_LEFT | wxEXPAND);
  
  m_pListBoxFilterMethod = new StringValueAndTitleListBox (this, ProcessSignal::getFilterMethodCount(), ProcessSignal::getFilterMethodTitleArray(), ProcessSignal::getFilterMethodNameArray());
  m_pListBoxFilterMethod->SetSelection (iDefaultFilterMethodID);
  pGridSizer->Add (new wxStaticText (this, -1, "Filter Method"), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
  pGridSizer->Add (m_pListBoxFilterMethod, 0, wxALL | wxALIGN_LEFT | wxEXPAND);
  
  m_pListBoxFilterGeneration = new StringValueAndTitleListBox (this, ProcessSignal::getFilterGenerationCount(), ProcessSignal::getFilterGenerationTitleArray(), ProcessSignal::getFilterGenerationNameArray());
  m_pListBoxFilterGeneration->SetSelection (iDefaultFilterGenerationID);
  pGridSizer->Add (new wxStaticText (this, -1, "Filter Generation"), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
  pGridSizer->Add (m_pListBoxFilterGeneration, 0, wxALL | wxALIGN_LEFT | wxEXPAND);
  
  
  m_pListBoxBackproject = new StringValueAndTitleListBox (this, Backprojector::getBackprojectCount(), Backprojector::getBackprojectTitleArray(), Backprojector::getBackprojectNameArray());
  m_pListBoxBackproject->SetSelection (iDefaultBackprojectID);
  pGridSizer->Add (new wxStaticText (this, -1, "Backprojection"), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
  pGridSizer->Add (m_pListBoxBackproject, 0, wxALL | wxALIGN_RIGHT | wxEXPAND);
  
  m_pListBoxInterp = new StringValueAndTitleListBox (this, Backprojector::getInterpCount(), Backprojector::getInterpTitleArray(), Backprojector::getInterpNameArray());
  m_pListBoxInterp->SetSelection (iDefaultInterpID);
  pGridSizer->Add (new wxStaticText (this, -1, "Interpolation"), 0, wxALIGN_CENTER_VERTICAL | wxALIGN_RIGHT | wxALL, 5);
  pGridSizer->Add (m_pListBoxInterp, 0, wxALL | wxALIGN_RIGHT | wxEXPAND);
  
  
  pGridSizer->Add (new wxStaticText (this, -1, "Trace Level"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  static const char* aszTraceTitle[] = {"None", "Full"};
  static const char* aszTraceName[] = {"none", "full"};
  m_pListBoxTrace = new StringValueAndTitleListBox (this, 2, aszTraceTitle, aszTraceName);
  iTrace = clamp(iTrace, 0, 1);
  m_pListBoxTrace->SetSelection (iTrace);
  pGridSizer->Add (m_pListBoxTrace);
  
  pGridSizer->Add (new wxStaticText (this, -1, "X Size"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlXSize, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Y Size"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlYSize, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Filter Parameter"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlFilterParam, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Zeropad"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlZeropad, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (new wxStaticText (this, -1, "Interpolation Parameter"), 0, wxALIGN_RIGHT | wxALIGN_CENTER_VERTICAL);
  pGridSizer->Add (m_pTextCtrlInterpParam, 0, wxALIGN_LEFT | wxALIGN_CENTER_VERTICAL);
  
  pTopSizer->Add (pGridSizer, 1, wxALL, 3);
  
  pTopSizer->Add (new wxStaticLine (this, -1, wxDefaultPosition, wxSize(3,3), wxHORIZONTAL), 0, wxEXPAND | wxALL, 5);
  
  wxBoxSizer* pButtonSizer = new wxBoxSizer (wxHORIZONTAL);
  wxButton* pButtonOk = new wxButton (this, wxID_OK, "Okay");
  wxButton* pButtonCancel = new wxButton (this, wxID_CANCEL, "Cancel");
  pButtonSizer->Add (pButtonOk, 0, wxEXPAND | wxALL, 10);
  pButtonSizer->Add (pButtonCancel, 0, wxEXPAND | wxALL, 10);
  
  pTopSizer->Add (pButtonSizer, 0, wxALIGN_CENTER);
  
  SetAutoLayout (true);
  SetSizer (pTopSizer);
  pTopSizer->Layout();
  pTopSizer->Fit (this);
  pTopSizer->SetSizeHints (this);
}

DialogGetReconstructionParameters::~DialogGetReconstructionParameters (void)
{
}


unsigned int
DialogGetReconstructionParameters::getXSize (void)
{
  wxString strCtrl = m_pTextCtrlXSize->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultXSize);
}

unsigned int
DialogGetReconstructionParameters::getYSize (void)
{
  wxString strCtrl = m_pTextCtrlYSize->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultYSize);
}

unsigned int
DialogGetReconstructionParameters::getZeropad (void)
{
  wxString strCtrl = m_pTextCtrlZeropad->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultZeropad);
}


unsigned int
DialogGetReconstructionParameters::getInterpParam (void)
{
  wxString strCtrl = m_pTextCtrlInterpParam->GetValue();
  unsigned long lValue;
  if (strCtrl.ToULong (&lValue))
    return lValue;
  else
    return (m_iDefaultInterpParam);
}

double
DialogGetReconstructionParameters::getFilterParam (void)
{
  wxString strCtrl = m_pTextCtrlFilterParam->GetValue();
  double dValue;
  if (strCtrl.ToDouble (&dValue))
    return (dValue);
  else
    return (m_dDefaultFilterParam);
}

const char*
DialogGetReconstructionParameters::getFilterName (void)
{
  return m_pListBoxFilter->getSelectionStringValue();
}

const char*
DialogGetReconstructionParameters::getFilterMethodName (void)
{
  return m_pListBoxFilterMethod->getSelectionStringValue();
}

const char*
DialogGetReconstructionParameters::getInterpName (void)
{
  return m_pListBoxInterp->getSelectionStringValue();
}

int
DialogGetReconstructionParameters::getTrace (void)
{
  int iTrace = 0;
  if (strcmp("full", m_pListBoxTrace->getSelectionStringValue()) == 0)
    iTrace = Trace::TRACE_PLOT;
  return iTrace;
}

const char*
DialogGetReconstructionParameters::getBackprojectName (void)
{
  return m_pListBoxBackproject->getSelectionStringValue();
}

const char*
DialogGetReconstructionParameters::getFilterGenerationName (void)
{
  return m_pListBoxFilterGeneration->getSelectionStringValue();
}


